
# q-expansion of newform 8424.2.a.n, downloaded from the LMFDB on 23 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8424
weight = 2
poly_data = [-3, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [1, 1], [0, -1], [-2, 1], [-1, 0], [-4, -2], [-1, 2], [1, -1], [9, 0], [2, -2], [6, -2], [-3, -3], [-2, -4], [0, 6], [1, 4], [-8, -3], [10, -3], [-4, 2], [5, 4], [-2, 2], [1, -5], [-4, -5], [-9, 5], [-1, -3], [-9, 4], [10, -4], [-9, -3], [-5, 3], [0, 7], [-14, 4], [-22, 0], [0, 2], [-9, 1], [-7, -3], [-16, -1], [16, -3], [-12, -2], [-9, 4], [-3, -10], [0, -4], [6, 11], [-6, -8], [-1, 9], [0, -6], [-14, -2], [7, -1], [10, 7], [-12, 9], [4, -2], [-2, 7], [5, -10], [3, -5], [-7, -7], [-10, 3], [-11, 5], [-10, 2], [8, 7], [16, -1], [-16, 6], [2, -16], [24, 4], [6, 12], [0, -2], [15, 4], [3, -7], [8, -2], [0, 0], [-9, 11], [18, -8], [-1, -5], [-6, 4], [-23, 1], [2, -15], [-13, 6], [-15, -8], [-26, 0], [2, 10], [-15, 7], [-6, -4], [15, 1], [5, 3], [-19, 4], [7, 10], [-18, -6], [-10, -2], [16, -4], [17, -9], [-23, -9], [-4, 19], [33, -1], [29, -4], [4, 3], [18, -6], [-15, -8], [3, 17], [-12, -4], [2, 3], [-11, 5], [9, -11], [8, 6], [-20, 0], [-5, -1], [4, -22], [-45, -1], [-4, 4], [16, 10], [-11, 5], [9, -17], [25, 6], [8, 0], [8, 12], [-27, -9], [-11, 12], [22, 15], [-8, -9], [-21, 8], [11, 7], [-12, 8], [0, 28], [8, 12], [25, -6], [19, -4], [-32, -10], [23, 2], [-3, -2], [-4, 8], [7, 9], [0, 24], [-1, 13], [-7, -2], [-5, 12], [-12, -10], [14, 5], [-16, 8], [-10, -22], [30, 6], [-11, -10], [1, -10], [-4, -19], [-5, -16], [12, -16], [-28, 6], [-40, 4], [22, -9], [0, -2], [11, -11], [0, 5], [-3, -25], [-17, -6], [-21, 5], [38, 0], [-31, 3], [23, -3], [10, 4], [22, 6], [4, -12], [-28, -10], [6, -2], [0, -14], [-30, 6], [-24, 8], [-28, 3], [33, 5], [39, 3], [-3, -12], [-11, 13], [-20, 3], [32, 2], [16, -4], [-30, 18], [42, 2], [-25, -5], [16, -12], [11, -13], [-28, -15], [32, 12], [-36, -10], [-17, -1], [20, 2], [-10, -9], [14, 0], [-18, 11], [30, 16], [15, -8], [-30, -6], [40, -12], [21, 18], [25, 17], [48, -2], [-2, 18], [-12, 5], [-17, -9], [-17, 5], [-15, -15], [8, -21], [4, -16], [52, -7], [-10, -6], [30, 16], [-10, -12], [-25, -3], [2, -16], [-14, -2], [-12, -31], [9, -26], [44, -12], [21, 7], [32, 2], [-8, 36], [9, 26], [-9, 28], [-23, 7], [2, 9], [41, 10], [-10, 2], [-13, -1], [-44, -7], [6, -8], [-12, 4], [8, 2], [0, 33], [-12, 16], [2, -17], [-9, -3], [44, 3], [12, 10], [-21, 11], [-3, 35], [-8, 8], [0, -18], [40, 10], [-28, -17], [-2, 39], [-39, 9], [-7, -36], [-16, 18], [40, -4], [-43, 9], [-14, 16], [18, 17], [-1, -3], [19, 23], [-5, 41], [24, 18], [-7, -23], [-32, 8], [0, -11], [37, -10], [56, 12], [-39, 17], [0, -20], [11, -26], [13, 12], [-45, 2], [-6, -33], [-40, -8], [22, -20], [69, 6], [-59, 9], [46, 4], [45, -19], [16, 16], [30, -19], [-23, -21], [-1, -7], [17, 5], [41, -15], [65, 2], [-10, -4], [22, 3], [-19, -6], [1, -11], [31, 15], [-29, -2], [20, 8], [-30, 29], [-19, 35], [25, 15], [-17, 17], [58, -8], [-8, 31], [-8, -4], [-25, 19], [-40, -2], [-2, 40], [11, -16], [18, 20], [26, 15], [-16, 23], [-23, 14], [33, 21], [31, -29], [24, -10], [72, -2], [31, 14], [26, 22], [33, 24], [39, -1], [-22, -8], [-13, 18], [-25, -7], [-28, -10], [25, 19], [12, -14], [-4, -46], [27, -33], [-4, -20], [-40, 0], [-16, -13], [-21, -21], [72, 2], [-22, -5], [-20, -34], [37, 8], [41, -20], [-12, -9], [-42, -12], [-73, -1], [60, 5], [25, 17], [-5, -25], [50, 0], [-26, -29], [43, 23], [-9, 30], [33, -10], [3, -22], [64, -14], [27, -6], [-29, 17], [-6, -26], [-18, 26], [0, 24], [15, 4], [-72, 0], [-5, 1], [-48, -12], [-13, 31], [-3, -45], [-23, -35], [12, 28], [-24, 2], [-15, 7], [10, 20], [-53, -19], [-30, 25], [13, -6], [9, 18], [-7, 0], [36, -8], [-25, -7], [-26, -20], [-9, -35], [2, -31], [2, -25], [43, 4], [-16, 9], [-38, -5], [69, -14], [41, 3], [24, 12], [-18, -8], [2, 9], [-11, 13], [-9, -4], [-21, 1], [-20, 4], [34, -30], [44, -18], [15, -17], [-49, 0], [46, -10], [49, 7], [32, -3], [72, 13], [-51, -3], [-8, -24], [6, -51], [47, 10], [-36, 8], [-10, 10], [18, -21], [-87, -3], [40, 12], [57, -6], [30, -5], [-28, -3], [-56, 4], [3, -37], [-68, -4], [58, 20], [-24, 21], [-30, -22], [-36, -6], [-47, -8], [-24, 25], [6, 39], [5, -11], [40, 22], [-7, -19], [36, -7], [-20, 4], [8, -42], [-20, 36], [-17, 45], [-11, 41], [27, -11], [-40, 20], [-42, -3], [-31, -10], [28, 4], [-13, -3], [-68, -12], [30, 14], [49, -24], [15, 40], [14, 28], [0, -22], [32, 14], [68, -4], [-70, 6], [-37, -24], [-34, -32], [-11, 47], [-79, 1], [4, 10], [-54, 0], [27, -42], [-5, -35], [-83, 7], [-42, 31], [-67, -3]]
